/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: Envelope.h,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */
#ifndef OpenSOAP_Envelope_H
#define OpenSOAP_Envelope_H

#include <OpenSOAP/Block.h>
#include <OpenSOAP/ByteArray.h>
#include <OpenSOAP/String.h>

#include <stdlib.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

    typedef struct tagOpenSOAPEnvelope OpenSOAPEnvelope;
    typedef OpenSOAPEnvelope    *OpenSOAPEnvelopePtr;

    int
    OPENSOAP_API
    OpenSOAPEnvelopeCreate(/* [out] */ OpenSOAPEnvelopePtr *soapEnv);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeCreateMB(/* [in]  */ const char *soapVer,
                             /* [in]  */ const char *envPrefix,
                             /* [out] */ OpenSOAPEnvelopePtr *soapEnv);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeCreateWC(/* [in]  */ const wchar_t *soapVer,
                             /* [in]  */ const wchar_t *envPrefix,
                             /* [out] */ OpenSOAPEnvelopePtr *soapEnv);

	int
	OPENSOAP_API
	OpenSOAPEnvelopeCreateString(/* [in]  */ OpenSOAPStringPtr soapVer,
								 /* [in]  */ OpenSOAPStringPtr envPrefix,
								 /* [out] */ OpenSOAPEnvelopePtr *soapEnv);
	
	int
	OPENSOAP_API
	OpenSOAPEnvelopeCreateFaultMB(/* [in]  */ const char *soapVer,
								  /* [in]  */ const char *envPrefix,
								  /* [in]  */ const char *faultCode,
								  /* [in]  */ const char *faultString,
								  /* [out] */ OpenSOAPBlockPtr *faultBlock,
								  /* [out] */ OpenSOAPEnvelopePtr *soapEnv);

	int
	OPENSOAP_API
	OpenSOAPEnvelopeCreateFaultWC(/* [in]  */ const wchar_t *soapVer,
								  /* [in]  */ const wchar_t *envPrefix,
								  /* [in]  */ const wchar_t *faultCode,
								  /* [in]  */ const wchar_t *faultString,
								  /* [out] */ OpenSOAPBlockPtr *faultBlock,
								  /* [out] */ OpenSOAPEnvelopePtr *soapEnv);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeCreateCharEncoding(/* [in]  */ const char *ch_enc,
                                       /* [in]  */ OpenSOAPByteArrayPtr b_arry,
                                       /* [out] */
									   OpenSOAPEnvelopePtr *soap_env);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeRetain(/* [in, out] */ OpenSOAPEnvelopePtr soap_env);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeRelease(/* [in, out] */ OpenSOAPEnvelopePtr soap_env);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeAddHeaderBlockMB(/* [in]  */ OpenSOAPEnvelopePtr soap_env,
                                     /* [in]  */ const char *block_name,
                                     /* [out] */ OpenSOAPBlockPtr *h_block);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeAddHeaderBlockWC(/* [in]  */ OpenSOAPEnvelopePtr soap_env,
                                     /* [in]  */ const wchar_t *block_name,
                                     /* [out] */ OpenSOAPBlockPtr *h_block);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeGetNextHeaderBlock(/* [in] */ OpenSOAPEnvelopePtr soap_env,
                                       /* [in, out] */
									   OpenSOAPBlockPtr *h_block);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeGetHeaderBlockMB(/* [in]  */ OpenSOAPEnvelopePtr soap_env,
                                     /* [in]  */ const char *block_name,
                                     /* [out] */ OpenSOAPBlockPtr *h_block);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeGetHeaderBlockWC(/* [in]  */ OpenSOAPEnvelopePtr soap_env,
                                     /* [in]  */ const wchar_t *block_name,
                                     /* [out] */ OpenSOAPBlockPtr *h_block);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeAddBodyBlockMB(/* [in]  */ OpenSOAPEnvelopePtr soap_env,
                                   /* [in]  */ const char *block_name,
                                   /* [out] */ OpenSOAPBlockPtr *b_block);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeAddBodyBlockWC(/* [in]  */ OpenSOAPEnvelopePtr soap_env,
                                   /* [in]  */ const wchar_t *block_name,
                                   /* [out] */ OpenSOAPBlockPtr *b_block);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeGetNextBodyBlock(/* [in]  */ OpenSOAPEnvelopePtr soap_env,
                                     /* [in, out] */
									 OpenSOAPBlockPtr *b_block);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeGetBodyBlockMB(/* [in]  */ OpenSOAPEnvelopePtr soap_env,
                                   /* [in]  */ const char *block_name,
                                   /* [out] */ OpenSOAPBlockPtr *b_block);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeGetBodyBlockWC(/* [in]  */ OpenSOAPEnvelopePtr soap_env,
                                   /* [in]  */ const wchar_t *block_name,
                                   /* [out] */ OpenSOAPBlockPtr *b_block);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeGetCharEncodingString(/* [in]  */
										  OpenSOAPEnvelopePtr soap_env,
                                          /* [in]  */ const char *ch_enc,
                                          /* [out] */
										  OpenSOAPByteArrayPtr b_ary);
    
    int
    OPENSOAP_API
    OpenSOAPEnvelopeGetHeaderCharEncodingString(/* [in]  */
												OpenSOAPEnvelopePtr soap_env,
											    /* [in]  */ const char *ch_enc,
											    /* [out] */
												OpenSOAPByteArrayPtr  b_ary);

    int
    OPENSOAP_API
    OpenSOAPEnvelopeGetBodyCharEncodingString(/* [in]  */
											  OpenSOAPEnvelopePtr soap_env,
											  /* [in]  */ const char *ch_enc,
											  /* [out] */
											  OpenSOAPByteArrayPtr b_ary);

	int
	OPENSOAP_API
	OpenSOAPEnvelopeAddFaultString(/* [out] */ OpenSOAPEnvelopePtr soapEnv,
								   /* [in]  */ OpenSOAPStringPtr faultCode,
								   /* [in]  */ OpenSOAPStringPtr faultString,
								   /* [in]  */ int isValueDup,
								   /* [out] */ OpenSOAPBlockPtr *faultBlock);
	
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* OpenSOAP_Envelope_H */
