/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: Serializer.h,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */
#ifndef OpenSOAP_Serializer_H
#define OpenSOAP_Serializer_H

#include <OpenSOAP/String.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

    /* OpenSOAP Serializer Function Type */
    typedef int
    (*OpenSOAPSerializerFunc)(/* [in]  */ void *from_value,
                              /* [out] */ OpenSOAPStringPtr to);

    /* OpenSOAP Deserializer Function Type */
    typedef int
    (*OpenSOAPDeserializerFunc)(/* [in]  */ OpenSOAPStringPtr from,
                                /* [out] */ void *to_value);


    /* Register Serializer */
    int
    OPENSOAP_API
    OpenSOAPSerializerRegisterMB(/* [in] */ const char *soap_typename,
                                 /* [in] */ OpenSOAPSerializerFunc serializer,
                                 /* [in] */ OpenSOAPDeserializerFunc deserializer);

    /* Register Serializer */
    int
    OPENSOAP_API
    OpenSOAPSerializerRegisterWC(/* [in] */ const wchar_t *soap_typename,
                                 /* [in] */ OpenSOAPSerializerFunc serializer,
                                 /* [in] */ OpenSOAPDeserializerFunc deserializer);

    /* Get Serializer(OpenSOAPString) */
    int
    OPENSOAP_API
    OpenSOAPGetSerializer(/* [in]  */ OpenSOAPStringPtr soap_typename,
                          /* [out] */ OpenSOAPSerializerFunc *serializer);

    /* Get Serializer(MB) */
    int
    OPENSOAP_API
    OpenSOAPGetSerializerMB(/* [in]  */ const char *soap_typename,
                            /* [out] */ OpenSOAPSerializerFunc *serializer);

    /* Get Serializer(WC) */
    int
    OPENSOAP_API
    OpenSOAPGetSerializerWC(/* [in]  */ const wchar_t *soap_typename,
                            /* [out] */ OpenSOAPSerializerFunc *serializer);
    
    /* Get Deserializer(OpenSOAPString) */
    int
    OPENSOAP_API
    OpenSOAPGetDeserializer(/* [in]  */ OpenSOAPStringPtr soap_typename,
                            /* [out] */ OpenSOAPDeserializerFunc *deserializer);
    /* Get Deserializer(MB) */
    int
    OPENSOAP_API
    OpenSOAPGetDeserializerMB(/* [in]  */ const char *soap_typename,
                              /* [out] */ OpenSOAPDeserializerFunc *deserializer);

    /* Get Deserializer(WC) */
    int
    OPENSOAP_API
    OpenSOAPGetDeserializerWC(/* [in]  */ const wchar_t *soap_typename,
                              /* [out] */ OpenSOAPDeserializerFunc *deserializer);

    /* default serializer regist */
    int
    OPENSOAP_API
    OpenSOAPSerializerRegistDefaults(void);
    
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* OpenSOAP_Serializer_H */
