/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: StringHash.h,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */
#ifndef OpenSOAP_StringHash_H
#define OpenSOAP_StringHash_H

#include <OpenSOAP/String.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

    typedef struct tagOpenSOAPStringHash OpenSOAPStringHash;
    typedef OpenSOAPStringHash    *OpenSOAPStringHashPtr;

    /* create string hash */
    int
    OPENSOAP_API
    OpenSOAPStringHashCreate(OpenSOAPStringHashPtr * /* [out] */ strh);

    /* release string hash */
    int
    OPENSOAP_API
    OpenSOAPStringHashRelease(OpenSOAPStringHashPtr  /* [in] */ strh);

    /* clear string hash */
    int
    OPENSOAP_API
    OpenSOAPStringHashClear(OpenSOAPStringHashPtr /* [in, out] */ strh);

    /* remove key */
    int
    OPENSOAP_API
    OpenSOAPStringHashRemoveKey(OpenSOAPStringHashPtr /* [in, out] */ strh,
                                OpenSOAPStringPtr /* [in] */ key,
                                void ** /* [out] */ val);
    
    /* set value */
    int
    OPENSOAP_API
    OpenSOAPStringHashSetValue(OpenSOAPStringHashPtr /* [in, out] */ strh,
                               OpenSOAPStringPtr /* [in] */ key,
                               void * /* [in] */ val);
    
    int
    OPENSOAP_API
    OpenSOAPStringHashSetValueMB(OpenSOAPStringHashPtr /* [in, out] */ strh,
                                 const char * /* [in] */ key,
                                 void * /* [in] */ val);

    int
    OPENSOAP_API
    OpenSOAPStringHashSetValueWC(OpenSOAPStringHashPtr /* [in, out] */ strh,
                                 const wchar_t * /* [in] */ key,
                                 void * /* [in] */ val);
    
    /* get value */
    int
    OPENSOAP_API
    OpenSOAPStringHashGetValue(OpenSOAPStringHashPtr /* [in] */ strh,
                               OpenSOAPStringPtr /* [in] */ key,
                               void ** /* [out] */ val);

    int
    OPENSOAP_API
    OpenSOAPStringHashGetValueMB(OpenSOAPStringHashPtr /* [in] */ strh,
                                 const char * /* [in] */ key,
                                 void ** /* [out] */ val);

    int
    OPENSOAP_API
    OpenSOAPStringHashGetValueWC(OpenSOAPStringHashPtr /* [in] */ strh,
                                 const wchar_t * /* [in] */ key,
                                 void ** /* [out] */ val);

    /* get size */
    int
    OPENSOAP_API
    OpenSOAPStringHashGetSize(OpenSOAPStringHashPtr /* [in] */ strh,
                              size_t * /* [out] */ sz);

    /* get keys */
    int
    OPENSOAP_API
    OpenSOAPStringHashGetKeys(OpenSOAPStringHashPtr /* [in] */ strh,
                              size_t * /* [in, out] */ sz,
                              OpenSOAPStringPtr * /* [out] */ keys);
    /* get values */
    int
    OPENSOAP_API
    OpenSOAPStringHashGetValues(OpenSOAPStringHashPtr /* [in] */ strh,
                                size_t * /* [in, out] */ sz,
                                void ** /* [out] */ vals);

    /* apply to all values */
    int
    OPENSOAP_API
    OpenSOAPStringHashApplyToValues(OpenSOAPStringHashPtr /* [in, out] */ strh,
                                    int  /* [in] */ (*aply)(void *, void *),
                                    void * /* [in] */ opt);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* OpenSOAP_StringHash_H */
