/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: Queue.h,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#ifndef QUEUE_H
#define QUEUE_H

#if defined(WIN32)
#include <windows.h>
#include <fstream>
#else
#include <pthread.h>
//#include <fstream.h>
#include <fstream>
#endif

#include <vector>
#include <string>

#define MAX_POP_THREAD_NUMBER 4
#define INITIAL_STATE 0
#define PROGRESS 1

namespace OpenSOAP {
  
  class ChannelDescriptor;
  
  class Queue {
  private:
#if defined(WIN32)
	HANDLE
#else
    pthread_t 
#endif
		pop_thread[MAX_POP_THREAD_NUMBER];

#if defined(WIN32)
	static int mtCount_;
	HANDLE
#else
    pthread_mutex_t 
#endif
		queue_mutex;

	//semaphore for windows
	//cond_wait for pthread
#if defined(WIN32)
	HANDLE queue_semaphore;
#else
    pthread_cond_t queue_cond;
#endif

    std::string tableFile;
    std::ifstream tin;
    std::ofstream tout;
    bool isFwdQueue_;
    struct TableRecord{
      //string msgID;
      std::string fileID;
      int proccessFlag; // INITIAL_STATE, PROGRESS
    };
    
    std::vector<TableRecord> queueTable;
    ChannelDescriptor chnlDesc_;  
  public:
    Queue(std::string filePass, bool isFwdQueue);
    int push(std::string fileID);
    ~Queue(); 
    void setChnlDesc(const ChannelDescriptor& chnlDesc);
    bool execPushOpr();
    
  private:
    int fileOut();
    static 
#if defined(WIN32)
	void
#else
	void* 
#endif
		pop(void* arg);

    bool popOutToNextDrv(std::string fileID);
    int queueEraseAndFileOut(std::string fileID);
    
  };
}
#endif /* QUEUE_H */
